"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useSSRComputation = void 0;
var babel_plugin_macros_1 = require("babel-plugin-macros");
var path = __importStar(require("path"));
var fs = __importStar(require("fs"));
var t = __importStar(require("@babel/types"));
var utils_1 = require("./utils");
function getProgramPath(nodePath) {
    var programPath = nodePath.findParent(function (path) { return path.isProgram(); });
    if (!programPath || !t.isProgram(programPath.node)) {
        throw new Error("Could not find program path");
    }
    return programPath;
}
function addImportStatement(importName, importPath, isDefault, nodePath) {
    var programPath = getProgramPath(nodePath);
    var existingImportDeclaration = programPath.node.body.find(function (p) {
        // return t.isImportDeclaration(p) && p.source.value === importPath;
        if (!t.isImportDeclaration(p) || p.source.value !== importPath) {
            return false;
        }
        if (isDefault) {
            return p.specifiers.some(function (specifier) { return t.isImportDefaultSpecifier(specifier) && specifier.local.name === importName; });
        }
        else {
            return p.specifiers.some(function (specifier) { return t.isImportSpecifier(specifier) && specifier.local.name === importName; });
        }
    });
    if (!existingImportDeclaration) {
        var importDeclarator = void 0;
        if (isDefault) {
            importDeclarator = t.importDefaultSpecifier(t.identifier(importName));
        }
        else {
            importDeclarator = t.importSpecifier(t.identifier(importName), t.identifier(importName));
        }
        var newImportDeclaration = t.importDeclaration([importDeclarator], t.stringLiteral(importPath));
        programPath.node.body.unshift(newImportDeclaration);
    }
}
var macro = function (_a) {
    var references = _a.references, state = _a.state;
    var currentFilename = state.file.opts.filename;
    if (!currentFilename) {
        throw new Error("useSSRComputation is called without filename");
    }
    var pluginOptions = state.opts;
    var opts = pluginOptions === null || pluginOptions === void 0 ? void 0 : pluginOptions.useSSRComputation;
    if (!opts || (opts.side !== 'client' && opts.side !== 'server')) {
        throw new Error("The \"side\" option must be specified in babel-plugin-macros config in babel.config.js:\n      plugins: [\n        [\n          \"macros\",\n          {\n            useSSRComputation: {\n              isSSRBundle ? \"server\" : \"client\",\n            },\n          },  \n\n      ]\n    ");
    }
    var side = opts === null || opts === void 0 ? void 0 : opts.side;
    (references.useSSRComputation || []).map(function (nodePath) {
        var parent = nodePath.parent;
        if (t.isCallExpression(parent)) {
            if (parent.arguments.length < 2) {
                throw new Error("useSSRComputation must be called with at least two arguments: a path to a .ssr-computation.js file containing the definition of the funciton and array of dependencies.");
            }
            if (parent.arguments.length > 3) {
                throw new Error("useSSRComputation must be called with at most three arguments: a path to a .ssr-computation.js file containing the definition of the funciton, array of dependencies and options object.");
            }
            var filenameNode = parent.arguments.shift();
            var optionsNode = parent.arguments.length === 2 ? parent.arguments.pop() : t.objectExpression([]);
            if (!t.isStringLiteral(filenameNode)) {
                throw new Error("The first argument must be a path to an existing ts file.");
            }
            // Check and parse options
            if (!t.isObjectExpression(optionsNode)) {
                throw new Error("The third argument must be an options object.");
            }
            var macroOptions = (0, utils_1.extractMacroOptions)(optionsNode);
            var webpackChunkName = (macroOptions.webpackChunkName ? macroOptions.webpackChunkName : 'default') + '-ssr-computations';
            var absolutePath_1 = path.resolve(path.dirname(currentFilename), filenameNode.value);
            var extensions = ['.ts', '.js', '.tsx', '.jsx'];
            if (!extensions.some(function (extension) { return fs.existsSync(absolutePath_1 + extension); })) {
                throw new Error("The file ".concat(filenameNode, "(.js/.ts/.jsx/.tsx) does not exist."));
            }
            var useSSRComputationFunctionName = "useSSRComputation_".concat(side.charAt(0).toUpperCase() + side.slice(1));
            parent.callee = t.identifier(useSSRComputationFunctionName);
            addImportStatement(useSSRComputationFunctionName, "@popmenu/use-ssr-computation.runtime/lib/".concat(useSSRComputationFunctionName), true, nodePath);
            if (side === 'server') {
                var importedFunctionName = filenameNode.value;
                var delimeter = '.ssr-computation';
                if (!importedFunctionName.endsWith('.ssr-computation')) {
                    throw new Error("The file ".concat(importedFunctionName, " must have the extension ").concat(delimeter, " to be used in useSSRComputation"));
                }
                importedFunctionName = importedFunctionName.replace(delimeter, '');
                importedFunctionName = importedFunctionName.replace(/[^a-zA-Z0-9]/g, '_');
                addImportStatement(importedFunctionName, filenameNode.value, true, nodePath);
                var identifier = t.identifier(importedFunctionName);
                parent.arguments.unshift(identifier);
            }
            else {
                nodePath.node.start;
                var importString = t.stringLiteral(filenameNode.value);
                importString.leadingComments = [
                    {
                        type: 'CommentBlock',
                        value: " webpackChunkName: \"".concat(webpackChunkName, "\" ")
                    }
                ];
                var dynamicImportFunctionName = nodePath.scope.generateUidIdentifier('dynamicImport_');
                var dynamicImportFunction = t.functionDeclaration(dynamicImportFunctionName, [], t.blockStatement([
                    t.returnStatement(t.callExpression(t.identifier('import'), [importString])),
                ]));
                var programPath = getProgramPath(nodePath);
                programPath.node.body.unshift(dynamicImportFunction);
                parent.arguments.unshift(dynamicImportFunctionName);
            }
            var relativePathToCwd = path.relative(process.cwd(), absolutePath_1);
            parent.arguments.push(optionsNode);
            parent.arguments.push(t.stringLiteral(relativePathToCwd));
        }
    });
};
exports.useSSRComputation = null;
exports.default = (0, babel_plugin_macros_1.createMacro)(macro, {
    configName: "useSSRComputation",
});
